package ui;

import java.lang.reflect.Constructor;

import ui.internal.ResourceMazeEditor;
import ui.internal.SolutionViewPanel;
import ui.internal.SolutionViewer;

/**
 * Main class for execution.
 * 
 * @author Petr
 */

public class Main {

	/**
	 * @param args
	 *            The must have one argument. It can be word 'editor' for
	 *            running maze editor or it can be class name of learning
	 *            algorithm with packages.
	 */

	public static void main(String[] args) {
		if (args.length != 1) {
			System.out.println("Spatny pocet parametru.");
			System.out
					.println("Metoda main ma mit jeden parametr, coz ma byt jmeno tridy, ktera obsahuje ucici algoritmus.");
			System.out
					.println("nebo parametr 'editor', cimz se spusti editor bludist");
			return;
		}

		if (args[0].equalsIgnoreCase("editor")) {
			new ResourceMazeEditor();
			return;
		}

		ClassLoader classLoader = ClassLoader.getSystemClassLoader();

		String className = args[0];

		try {
			Class cl = classLoader.loadClass(className);
			Constructor construct = cl.getConstructor();

			Object instance = construct.newInstance();

			if (instance instanceof SolutionInterface) {
				SolutionInterface solution = (SolutionInterface) instance;
				new SolutionViewer(new SolutionViewPanel(solution));
			}
		} catch (Exception e) {
			System.out
					.println("Nepodarilo se nahrat algoritmus z duvodu: " + e);
			System.out.println();
			System.out
					.println("Jmeno tridy je treba zadavat vcetne balicku, napr. ui.MujAlgoritmus");
			System.out
					.println("a takovato trida musi implementovat rozhrani ui.SolutionInterface.");
		}
	}

}
