package ui.internal;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.List;

import ui.Action;
import ui.Environment;
import ui.SolutionInterface;
import ui.StudentStateInterface;

public class SolutionViewPanel extends MazeViewPanel {
	
	private SolutionInterface solution;
	
	private Environment environment;

	public boolean drawActions = true;
	public boolean drawValues = true;
	public boolean drawGoals = false;
	public boolean drawRewards = false;
	public boolean drawTeleports = false;
	public boolean drawWalls = true;
	
	
	public SolutionViewPanel(SolutionInterface solution) {
		this.solution = solution;
	}
	
	
	public void setEnvironment(Environment environment) {
		this.environment = environment;
		solution.solveMDP(environment);
	}
	
	
	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		Graphics2D g2d = (Graphics2D)g;
	}
	
	protected void drawActions(Graphics2D g2d) {
		if (!drawActions) {
			return;
		}
		
		Maze maze = this.getMaze();
		
		float wallWidth = this.getWallWidth();
		int cellSize = this.getNodeLength();
		
		
		int arrowLenght = (int)(cellSize*0.5-wallWidth/2-1);
		int subArrowLenght = (int)(arrowLenght*0.333);
		
		
		for (StudentStateInterface state : environment.getAllStates()) {
			Color arrowColor = Color.RED;
			
			Action action = solution.getActionForState(state);
			
			StateInterface s = (StateInterface)state;
			
			if (!s.isDummyState()) {
				int x = s.getX();
				int y = s.getY();
				
				int fromX = x*cellSize+cellSize/2;
				int fromY = y*cellSize+cellSize/2;
				
				if (Action.GO_SOUTH.equals(action)) {
					drawArrowDown(g2d, arrowColor, arrowLenght, subArrowLenght, fromX, fromY);
				}
				if (Action.GO_NORTH.equals(action)) {
					drawArrowUp(g2d, arrowColor, arrowLenght, subArrowLenght, fromX, fromY);
				}
				if (Action.GO_WEST.equals(action)) {
					drawArrowLeft(g2d, arrowColor, arrowLenght, subArrowLenght, fromX, fromY);
				}
				if (Action.GO_EAST.equals(action)) {
					drawArrowRight(g2d, arrowColor, arrowLenght, subArrowLenght, fromX, fromY);
				}
			}
		}
	}
	
	
	private void drawArrowDown(Graphics2D g2d, Color arrowColor, int arrowLenght, int subArrowLenght, int fromX, int fromY) {
		Color oldColor = g2d.getColor();
		g2d.setColor(arrowColor);

		int xTo = fromX; 
		int yTo = fromY+arrowLenght;
		g2d.drawLine(fromX, fromY, xTo, yTo);
		
		g2d.drawLine(xTo-subArrowLenght, yTo-subArrowLenght, xTo, yTo);
		g2d.drawLine(xTo+subArrowLenght, yTo-subArrowLenght, xTo, yTo);
		
		g2d.setColor(oldColor);
	}
	
	private void drawArrowUp(Graphics2D g2d, Color arrowColor, int arrowLenght, int subArrowLenght, int fromX, int fromY) {
		Color oldColor = g2d.getColor();
		g2d.setColor(arrowColor);

		int xTo = fromX; 
		int yTo = fromY-arrowLenght;
		g2d.drawLine(fromX, fromY, xTo, yTo);
		
		g2d.drawLine(xTo-subArrowLenght, yTo+subArrowLenght, xTo, yTo);
		g2d.drawLine(xTo+subArrowLenght, yTo+subArrowLenght, xTo, yTo);
		
		g2d.setColor(oldColor);
		
	}
	
	private void drawArrowLeft(Graphics2D g2d, Color arrowColor, int arrowLenght, int subArrowLenght, int fromX, int fromY) {
		
		Color oldColor = g2d.getColor();
		g2d.setColor(arrowColor);

		
		int xTo = fromX-arrowLenght; 
		int yTo = fromY;
		g2d.drawLine(fromX, fromY, xTo, yTo);
		
		g2d.drawLine(xTo+subArrowLenght, yTo-subArrowLenght, xTo, yTo);
		g2d.drawLine(xTo+subArrowLenght, yTo+subArrowLenght, xTo, yTo);

		g2d.setColor(oldColor);
	}
	
	private void drawArrowRight(Graphics2D g2d, Color arrowColor, int arrowLenght, int subArrowLenght, int fromX, int fromY) {
		Color oldColor = g2d.getColor();
		g2d.setColor(arrowColor);
		
		int xTo = fromX+arrowLenght; 
		int yTo = fromY;
		g2d.drawLine(fromX, fromY, xTo, yTo);
		
		g2d.drawLine(xTo-subArrowLenght, yTo-subArrowLenght, xTo, yTo);
		g2d.drawLine(xTo-subArrowLenght, yTo+subArrowLenght, xTo, yTo);
	
		g2d.setColor(oldColor);
	}
	
	
	public void setDrawWalls(boolean drawWalls) {
		this.drawWalls = drawWalls;
		repaint();
	}
	
	
	private double getMinimalValue(List<StudentStateInterface> states) {
		double min = Double.MAX_VALUE;
		
		for (StudentStateInterface state : states) {
			StateInterface s = (StateInterface)state;
			
			if (!s.isDummyState()) {
				double value = solution.getValueForState(state);
				if (value < min) {
					min = value;
				}
			}
		}
		return min;
	}
	
	private double getMaximalValue(List<StudentStateInterface> states) {
		double max = Double.MIN_VALUE;
		
		for (StudentStateInterface state : states) {
			StateInterface s = (StateInterface)state;
			
			if (!s.isDummyState()) {
				double value = solution.getValueForState(state);
				if (value > max) {
					max = value;
				}
			}
		}
		
		return max;
	}
	
	
	protected void drawValues(Graphics2D g2d) {
		if (!drawValues) {
			return;
		}
		
		List<StudentStateInterface> states = environment.getAllStates();
		
		if (states == null) {
			return;
		}
		
		
		double min = getMinimalValue(states);
		double max = getMaximalValue(states);
		
		
		for (StudentStateInterface state : states) {
			
			StateInterface s  = (StateInterface)state;
			
			if (!s.isDummyState()) {
				int x = s.getX();
				int y = s.getY();
				
				double value = solution.getValueForState(state);
				
	            int red = 155-Math.min((int)(255.0*(value-min)/(max-min)),155);
	            int green = 155-Math.min((int)(255.0*(value-min)/(max-min)),155);
	            int blue = 255-Math.min((int)(255.0*(value-min)/(max-min)),220);
	            
	            Color fillColor = new Color(red, green, blue);
	            this.fillCellByColor(g2d, x, y, fillColor);
	            
	            Color textColor = Color.WHITE;
	            
	            this.drawText(g2d, x, y, textColor, value+"");
			}
		}
	} 
	
	
	@Override
	protected void drawGoals(Graphics2D g) {
		if (drawGoals) {
			super.drawGoals(g);
		}
	}
	@Override
	protected void drawRewards(Graphics2D g2d) {
		if (drawRewards) {
			super.drawRewards(g2d);
		}

	}
	@Override
	protected void drawTeleports(Graphics2D g) {
		if (drawTeleports) {
			super.drawTeleports(g); 
		}
	}
	@Override
	protected void drawWalls(Graphics g) {
		if (drawWalls) {
			super.drawWalls(g);
		}
	}
	
	
	public void setDrawActions(boolean drawActions) {
		this.drawActions = drawActions;
		repaint();
	}
	
	public void setDrawValues(boolean drawValues) {
		this.drawValues = drawValues;
		repaint();
	}
	public void setDrawGoals(boolean drawGoals) {
		this.drawGoals = drawGoals;
		repaint();
	}
	
	public void setDrawRewards(boolean drawRewards) {
		this.drawRewards = drawRewards;
		repaint();
	}

	public void setDrawTeleports(boolean teleports) {
		this.drawTeleports = teleports;
		repaint();
	}

}
